      subroutine diagon(njx,nqmax,nrootx,nex,ned,nev,numar,numarev,ntst
     &,vp,vs,vv)
C**********************************************************************
c*   Subrutina para diagonalizar los hamiltonianos:                   *
c*     H(S+,S_,S0)= 2\epsilon S_0 + \omega_b b^t b +                  *
c*     V_1*( vp*(S+^2 b   +  S-^2b^t)  Pair creation and annihilation *
c*          +vs*(2*S+S-(b + b^t)       Scattering                     * 
c*          +vv*(S+^2 b^t + S-^2 b) )  Vacuum corelations             *
c*                                                                    *
c*      y  modelos bosonicos                                          *
c*                                                                    *
C*   para valores fijos de pseudoespin (2*j=ndj) y paridad.           *
C*                                                                    *
c*   Variables de entrada:                                            *
c*      -Variables mudas                                              *
c*        njx : no. de renglones de la matriz simetrica.              *
c*        nqmax: Numero maximo de pares de cuarks       
c*        nrootx: numero de eigenvalores que se piden.                *
c*        nex: 0 paridad positiva, 2 paridad negativa                 *
c*        ned: 1 modelo bosonico,2 mod. bos. con corte,cualquier otro *
c*            numero modelo exacto.                                   *
c*        nev: 1 calcular valores de expectacion, cualquier otro para *
c*              NO calcularlos                                        *
c*        numar: numero del archivo en donde escribir energias        *
c*        numarev:numero de archivo en donde escribir valores de expec*
c*        ntst: Controla la manera en como  escribir los eigenvalores *
c*                en el archivo de salida:                            *
c*              1 lista de eigenvalores (para calc fun de particion)  *
c*              0 eigenvalores versus V1(para analisis de fases a T=0)*
c*       Pesos para los difrentes terminos de interaccion:            * 
c*        vp: Pair creation and annihilation                          *
c*        vs: Scattering                                              *
c*        vv: Vacuum corelations                                      *
c*                                                                    *
c*      -EN SENTENCIAS  common:                                       *
c*        parametros del modelo:epsilon,omegab,v1,omega               *
c*        numero maximo de gluones: ngmax                             *
c*        2*pseudoespin: ndj=2*J                                      *
c*                                                                    *
C*   Variables de salida:                                             *
c*      - En sentencias common                                        *
c*        e0: ENERGIA DEL VACIO (MINIMO DEL SECTOR J=omega,PARIDAD +) *
c*      - A escribir en archivos                                      *
c*        e0:  (de nuevo)                                             *
c*        root(nn): eigenvalores                                      *
c*        e()     : eigenvalores respecto al vacio (e0)               *
c*        xnq00: valores de expectacion fermionicos                   *
c*        xng0 : valores de expectacion bosonicos                     *
c*                                                                    *
C*    Variables internas                                              *
c*         A(): arreglo unidimensional de la matriz a proporcionar    *
c*              a subrutina GIVENS.FOR                                *
c*         h:  elemento de matriz                                     *
c**********************************************************************
c*    10 junio 2002 Sergio Lerma                                      *
c**********************************************************************
      integer njx,nrootx,nex,ned,nev,numar,numarev,ntst,omega,ngmax,ndj
      real epsilon,omegab,v1,e0,root(nrootx),vect(njx,nrootx),h,
     &A(((njx+1)*njx)/2), xnq00(nrootx),e(nrootx),nq00,
     &xng0(nrootx),ng0,vc
c*************SENTENCIAS COMMON****************************************
      common /parametros/epsilon,omegab,v1,e0
      common /topes/omega,ngmin,ngmax,ndj
c**********************************************************************
c  construyendo matriz para J dado (2J=ndj).
      if (ned.eq.1) vc=0.0 !Modelo bosonico
      if (ned.eq.2) vc=1.0 !Modelo bosonico con corte
      i=0
c---columnas de la matriz
      do na=ngmin,ngmax           !numero de gluones
      do ma=-ndj+nex,min(-2*omega+2*nqmax,ndj),4!ma=2*m con m=proyeccion
                                                ! del pseudoespin
c---renglones de la matriz         
       do nb=ngmin,na             !numero de gluones
       mblim=min(-2*omega+2*nqmax,ndj)              
       if (nb.eq.na) mblim=ma
       do mb=-ndj+nex,mblim,4 !ma=2*m con m=proyeccion del pseudoespin
         i=i+1
c  calculamos elementos de matriz:
         h=0.0
c-------------DIAGONAL-------------------------------------------------
         if (na.eq.nb .and. ma.eq.mb)  h=epsilon*ma+omegab*na
     &                                   +2*omega*epsilon
c-------------INTERACTION TERMS----------------------------------------
         if (nb.eq.(na-1)) then
c-------------PAIR CREATION AND ANNIHILATION---------------------------
c  S+^2 b:
          if (mb.eq.(ma+4) ) then 
           if (ned .ne. 1 .and. ned.ne.2) then                 !EXACTO
            h=vp*(v1/4.0)*sqrt(1.0*na)*sqrt(real( (ndj*(ndj+2) !EXACTO 
     &         -ma*(ma+2))*(ndj*(ndj+2)-(ma+2)*(ma+4)) ) )     !EXACTO
           else
            h=vp*2.0*(v1/4.0)*sqrt(1.0*na)*ndj            !MOD BOSONICO
     &        *sqrt(1.0*(ma+ndj+4)*(ma+ndj+2))            !MOD BOSONICO
     &        *(1.0-vc*(ma+ndj)/(2*ndj))                  !CON CORTE 
           end if  
           
          end if 
c  S-^2 b^t no cuenta
c-------------SCATTERING-----------------------------------------------
c  S+S- b:
          if (mb.eq.ma) then
           if (ned .ne. 1 .and. ned.ne.2) then   
            h=vs*(v1/2.0)*sqrt(1.0*na)*(ndj*(ndj+2)-ma*(ma-2))!EXACTO
           else
            h=vs*4.0*(v1/4.0)*sqrt(1.0*na)*ndj*(ma+ndj)  !MOD BOSONICO
     &        *(1-vc*(ma+ndj)/(2*ndj))                   !CON CORTE
           end if   
          end if   
c  S+S- b^t no cuenta
c-------------VACUUM CORELATIONS---------------------------------------
c  S-^2 b: 
          if (mb.eq.(ma-4)) then
           if (ned .ne. 1 .and. ned.ne.2) then
            h=vv*(v1/4.0)*sqrt(1.0*na)*sqrt(real ( (ndj*(ndj+2)!EXACTO
     &        -ma*(ma-2))*(ndj*(ndj+2)-(ma-2)*(ma-4)) ) )
           else
            h=vv*2.0*(v1/4.0)*sqrt(1.0*na)*ndj            !MOD BOSONICO
     &        *sqrt(1.0*(mb+ndj+4)*(mb+ndj+2))            !MOD BOSONICO
     &        *(1.0-vc*(ma+ndj)/(2*ndj))                  !CON CORTE 
           end if    
          end if  
c  S+^2 b^t no cuenta
         end if
c-----------------------------------------------------------------------
c Asigna valor de elemento de matriz a arreglo A()
         a(i)=h            
       end do
       end do
      end do
      end do
      if (i.ne.((njx+1)*njx)/2) then
         write(*,*) "error!!!, tamanio de matriz (",i,") no concuerda",
     &              " con tamanio declarado (",((njx+1)*njx)/2,")"
         stop
      end if
c llamada a subrutina de diagonalizacion
      CALL  GIVENS(A,ROOT,VECT,NJX,NROOTX,NJX)
c CALCULO VALORES DE EXPECTACION 
      if (nev.eq.1) then ! Solo si  se quieren calcular.
       do mn=nrootx,1,-1
         i=0
         nq00=0.0
         ng0=0.0
         do na=0,ngmax
          do ma=-ndj+nex,min(-2*omega+2*nqmax,ndj),4
            i=i+1
            nq00=nq00+((1.0*ma)/2 + omega)*(vect(i,mn)**2) 
            ng0 =ng0 +1.0*na*(vect(i,mn)**2) 
          end do
         end do
         xnq00(nrootx+1-mn)=nq00
         xng0(nrootx+1-mn)=ng0
       end do
      end if   
      if (nex.eq.0 .and. ndj.eq.2*omega) then
         e0=root(nrootx)        !Energia del vacio (pseudoJ=0,Par +)
         if (ntst.eq.1) write(numar,560) e0
      end if   
c_______________________SALIDA_________________________________________
C LISTA  DE ENERGIAS PARA CALCULAR FUN DE PARTICION
      if (ntst.eq.1) then
       write(numar,*) nrootx,nex,ndj !# de eigenvalores , 0:P+ o 2:P-, 2J 
       if (nev.eq.1) write(numarev,*) nrootx,nex,ndj
       do nn=1,nrootx!!!AQUI,1,-1
         write(numar,560) root(nrootx+1-nn)  -e0         
         if (nev.eq.1) 
     &   write(numarev,560) xnq00(nn),xng0(nn) !<nq+nq->/2 , <n_2g>
       end do
      end if
C ENERGIAS VS V1
      if (ntst.eq.0) then
       do nn=nrootx,1,-1
         e(nrootx+1-nn)=root(nn)-e0
       end do
c Se escriben energias (maximo 7)
       write(numar,560) v1,e0,e(1),e(2),e(3),e(4),e(5),e(6),e(7)
c     &                  ,e(8),e(9),e(10)
c Se escriben valores de expectacion
       if (nev.eq.1) 
     & write(numarev,560) v1,xnq00(1),xnq00(2),xnq00(3),xnq00(4)
     &,xnq00(5)
c     &,xnq00(6),xnq00(7),xnq00(8),xnq00(9),xnq00(10)
     &,xng0(1) ,xng0(2),xng0(3),xng0(4),xng0(5)
c     &,xng0(6),xng0(7),xng0(8),xng0(9) ,xng0(10) 
      end if 
560   format(1x,16(1x,f12.7))
      return 
      end
c**********************************************************************
c FIN DE SUBRUTINA DIAGON                                             *
c**********************************************************************







